"use strict";
/*
 * Copyright (c) 2020 Textlint Plugin LaTex2e team. All rights reserved.
 *
 * This file is part of textlint-plugin-latex2e.
 * This software is released under the MIT License, see LICENSE.md .
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ASTTester = __importStar(require("@textlint/ast-tester"));
require("jest");
const ast_node_types_1 = require("@textlint/ast-node-types");
const completeComment_1 = require("../src/completeComment");
const makeLocation = (startLine, startColumn, endLine, endColumn) => {
    return {
        start: {
            line: startLine,
            column: startColumn,
        },
        end: {
            line: endLine,
            column: endColumn,
        },
    };
};
const makeRange = (start, end) => {
    return [start, end];
};
describe("Conversion of latexParser.Comment", () => {
    test("convert valid", () => __awaiter(void 0, void 0, void 0, function* () {
        const rawText = "\\begin{document}\n% comment\n\\end{document}";
        const comments = [
            {
                kind: "comment",
                content: " comment",
                location: {
                    start: {
                        offset: 17,
                        line: 2,
                        column: 1,
                    },
                    end: {
                        offset: 27,
                        line: 3,
                        column: 1,
                    },
                },
            },
        ];
        const expected = [
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                loc: makeLocation(2, 0, 3, 0),
                range: makeRange(17, 27),
                raw: "% comment\n",
                value: " comment",
            },
        ];
        const actual = completeComment_1.convertCommentToTxtNode(rawText, comments);
        expect(actual).toMatchObject(expected);
        ASTTester.test(actual[0]);
    }));
    test("undefined", () => __awaiter(void 0, void 0, void 0, function* () {
        const actual = completeComment_1.convertCommentToTxtNode("", undefined);
        expect(actual).toMatchObject([]);
    }));
    test("null", () => __awaiter(void 0, void 0, void 0, function* () {
        const actual = completeComment_1.convertCommentToTxtNode("", null);
        expect(actual).toMatchObject([]);
    }));
});
describe("Whether the comment is appeared before the node", () => {
    test("the comment before the node", () => __awaiter(void 0, void 0, void 0, function* () {
        // % comment
        //
        // \begin{document}
        // \end{document}
        const commentRange = makeRange(0, 10);
        const nodeRange = makeRange(11, 42);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeTruthy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeFalsy();
    }));
    test("just before the node", () => __awaiter(void 0, void 0, void 0, function* () {
        // % comment
        // \begin{ document }
        // \end{document}
        const commentRange = makeRange(0, 10);
        const nodeRange = makeRange(10, 41);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeTruthy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeFalsy();
    }));
    test("the comment after the node", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        // \end{document}
        //
        // % comment
        const commentRange = makeRange(33, 43);
        const nodeRange = makeRange(0, 31);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeFalsy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeFalsy();
    }));
    test("just after the node", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        // \end{document}% comment
        const commentRange = makeRange(31, 41);
        const nodeRange = makeRange(0, 31);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeFalsy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeFalsy();
    }));
    test("the comment included by the node", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        //
        // % comment
        //
        // \end{document}
        const commentRange = makeRange(18, 28);
        const nodeRange = makeRange(0, 43);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeFalsy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeTruthy();
    }));
    test("the comment included by the node (on the same line)", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}% comment
        // \end{document}
        const commentRange = makeRange(16, 26);
        const nodeRange = makeRange(0, 40);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeFalsy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeTruthy();
    }));
    test("the comment included by the node", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        // % comment
        // \end{document}
        const commentRange = makeRange(17, 27);
        const nodeRange = makeRange(0, 41);
        expect(completeComment_1.isAppearedBeforeNode(nodeRange, commentRange)).toBeFalsy();
        expect(completeComment_1.isIncludedByNode(nodeRange, commentRange)).toBeTruthy();
    }));
});
describe("Test isParentNode", () => {
    test("valid", () => __awaiter(void 0, void 0, void 0, function* () {
        const node = {
            type: ast_node_types_1.ASTNodeTypes.Document,
            raw: "",
            range: [0, 1],
            loc: makeLocation(0, 0, 0, 0),
            children: [],
        };
        expect(completeComment_1.isParentNode(node)).toBeTruthy();
    }));
    test("null", () => __awaiter(void 0, void 0, void 0, function* () {
        const node = {
            type: ast_node_types_1.ASTNodeTypes.Document,
            raw: "",
            range: [0, 1],
            loc: makeLocation(0, 0, 0, 0),
            children: null,
        };
        expect(completeComment_1.isParentNode(node)).toBeFalsy();
    }));
    test("false", () => __awaiter(void 0, void 0, void 0, function* () {
        const node = {
            type: ast_node_types_1.ASTNodeTypes.Comment,
            raw: "",
            range: [0, 1],
            loc: makeLocation(0, 0, 0, 0),
            value: "",
        };
        expect(completeComment_1.isParentNode(node)).toBeFalsy();
    }));
});
describe("Test insertComment", () => {
    test("flat nodes", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        // abcd
        // % comment
        // efgh
        // \end{document}
        const nodes = [
            {
                type: ast_node_types_1.ASTNodeTypes.Document,
                range: makeRange(0, 52),
                loc: makeLocation(1, 0, 5, 14),
                raw: "",
                children: [
                    {
                        type: ast_node_types_1.ASTNodeTypes.Str,
                        range: makeRange(17, 21),
                        loc: makeLocation(2, 0, 2, 4),
                        raw: "abcd",
                        value: "abcd",
                    },
                    {
                        type: ast_node_types_1.ASTNodeTypes.Str,
                        range: makeRange(33, 37),
                        loc: makeLocation(4, 0, 4, 4),
                        raw: "efgh",
                        value: "efgh",
                    },
                ],
            },
        ];
        const comment = {
            type: ast_node_types_1.ASTNodeTypes.Comment,
            range: makeRange(22, 23),
            loc: makeLocation(3, 0, 4, 0),
            raw: "% comment\n",
            value: " comment",
        };
        const expected = JSON.parse(JSON.stringify(nodes));
        expected[0].children.splice(1, 0, comment);
        const actual = completeComment_1.insertComment(comment, nodes);
        expect(actual).toMatchObject(expected);
        for (const node of actual) {
            ASTTester.test(node);
        }
    }));
    test("nested nodes", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        // \begin{itemize}
        //     \item \textit{item1} % comment
        //     \item item2
        // \end{itemize}
        // \end{document}
        const nodes = [
            {
                type: ast_node_types_1.ASTNodeTypes.Document,
                range: [0, 112],
                loc: makeLocation(1, 0, 6, 14),
                raw: "",
                children: [
                    {
                        type: ast_node_types_1.ASTNodeTypes.Paragraph,
                        range: makeRange(17, 97),
                        loc: makeLocation(2, 0, 5, 13),
                        raw: "",
                        children: [
                            {
                                type: ast_node_types_1.ASTNodeTypes.Emphasis,
                                range: makeRange(43, 57),
                                loc: makeLocation(3, 10, 3, 24),
                                raw: "",
                                children: [
                                    {
                                        type: ast_node_types_1.ASTNodeTypes.Str,
                                        range: makeRange(51, 56),
                                        loc: makeLocation(3, 18, 3, 23),
                                        raw: "",
                                        value: "item1",
                                    },
                                ],
                            },
                        ],
                    },
                    {
                        type: ast_node_types_1.ASTNodeTypes.Str,
                        range: makeRange(78, 83),
                        loc: makeLocation(4, 10, 4, 15),
                        raw: "",
                        value: "items",
                    },
                ],
            },
        ];
        const comment = {
            type: ast_node_types_1.ASTNodeTypes.Comment,
            range: makeRange(58, 68),
            loc: makeLocation(3, 25, 4, 0),
            raw: "% comment\n",
            value: " comment",
        };
        const expected = JSON.parse(JSON.stringify(nodes));
        expected[0].children[0].children.push(comment);
        const actual = completeComment_1.insertComment(comment, nodes);
        expect(actual).toMatchObject(expected);
        for (const node of actual) {
            ASTTester.test(node);
        }
    }));
    test("comment before document nodes", () => __awaiter(void 0, void 0, void 0, function* () {
        // % comment
        // \begin{document}
        // \end{document}
        const nodes = [
            {
                type: ast_node_types_1.ASTNodeTypes.Document,
                range: makeRange(10, 41),
                loc: makeLocation(2, 0, 3, 14),
                raw: "",
                children: [],
            },
        ];
        const comment = {
            type: ast_node_types_1.ASTNodeTypes.Comment,
            range: makeRange(0, 10),
            loc: makeLocation(1, 0, 2, 0),
            raw: "% comment\n",
            value: " comment",
        };
        const expected = JSON.parse(JSON.stringify(nodes));
        expected.splice(0, 0, comment);
        const actual = completeComment_1.insertComment(comment, nodes);
        expect(actual).toMatchObject(expected);
        for (const node of actual) {
            ASTTester.test(node);
        }
    }));
    test("comment after document nodes", () => __awaiter(void 0, void 0, void 0, function* () {
        // \begin{document}
        // \end{document}
        // % comment
        const nodes = [
            {
                type: ast_node_types_1.ASTNodeTypes.Document,
                range: makeRange(0, 31),
                loc: makeLocation(1, 0, 2, 14),
                raw: "",
                children: [],
            },
        ];
        const comment = {
            type: ast_node_types_1.ASTNodeTypes.Comment,
            range: makeRange(32, 42),
            loc: makeLocation(3, 0, 4, 0),
            raw: "% comment\n",
            value: " comment",
        };
        const expected = JSON.parse(JSON.stringify(nodes));
        expected.push(comment);
        const actual = completeComment_1.insertComment(comment, nodes);
        expect(actual).toMatchObject(expected);
        for (const node of actual) {
            ASTTester.test(node);
        }
    }));
    test("comment after text without any white space", () => __awaiter(void 0, void 0, void 0, function* () {
        // https://github.com/textlint/textlint-plugin-latex2e/issues/52
        // A%B
        // C
        const comment = {
            type: ast_node_types_1.ASTNodeTypes.Comment,
            range: makeRange(1, 4),
            loc: makeLocation(1, 2, 2, 1),
            raw: "%B\n",
            value: "B",
        };
        const nodes = [
            {
                type: ast_node_types_1.ASTNodeTypes.Str,
                range: makeRange(0, 1),
                loc: makeLocation(1, 1, 2, 2),
                raw: "A",
                value: "A",
            },
            {
                type: ast_node_types_1.ASTNodeTypes.Str,
                range: makeRange(4, 5),
                loc: makeLocation(2, 1, 2, 2),
                raw: "C",
                value: "C",
            },
        ];
        const expected = JSON.parse(JSON.stringify(nodes));
        expected.splice(1, 0, comment);
        const actual = completeComment_1.insertComment(comment, nodes);
        expect(actual).toMatchObject(expected);
        for (const node of actual) {
            ASTTester.test(node);
        }
    }));
});
describe("Test completeComment", () => {
    test("One comment", () => __awaiter(void 0, void 0, void 0, function* () {
        const rawText = "\\begin{document}\n% comment\n\\end{document}";
        const nodes = {
            type: ast_node_types_1.ASTNodeTypes.Document,
            range: makeRange(0, 52),
            loc: makeLocation(1, 0, 3, 14),
            raw: rawText,
            children: [],
        };
        const comments = [
            {
                kind: "comment",
                content: " comment",
                location: {
                    start: {
                        offset: 17,
                        line: 2,
                        column: 1,
                    },
                    end: {
                        offset: 27,
                        line: 3,
                        column: 1,
                    },
                },
            },
        ];
        const expectedComments = [
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(17, 27),
                loc: makeLocation(2, 0, 3, 0),
                raw: "% comment\n",
                value: " comment",
            },
        ];
        const expected = JSON.parse(JSON.stringify(nodes));
        expected.children = [expectedComments[0]];
        const actual = completeComment_1.completeComments(comments)(rawText)(nodes);
        expect(actual).toMatchObject(expected);
        ASTTester.test(actual);
    }));
    test("Multi comments", () => __awaiter(void 0, void 0, void 0, function* () {
        const rawText = "% a\n% b\n\\begin{document}\n% c\n% d\n\\end{document}\n% e\n% f";
        const nodes = {
            type: ast_node_types_1.ASTNodeTypes.Document,
            range: makeRange(0, 56),
            loc: makeLocation(3, 0, 7, 0),
            raw: rawText,
            children: [],
        };
        const comments = [
            {
                kind: "comment",
                content: " a",
                location: {
                    start: {
                        offset: 0,
                        line: 1,
                        column: 1,
                    },
                    end: {
                        offset: 4,
                        line: 2,
                        column: 1,
                    },
                },
            },
            {
                kind: "comment",
                content: " b",
                location: {
                    start: {
                        offset: 4,
                        line: 2,
                        column: 1,
                    },
                    end: {
                        offset: 8,
                        line: 3,
                        column: 1,
                    },
                },
            },
            {
                kind: "comment",
                content: " c",
                location: {
                    start: {
                        offset: 25,
                        line: 4,
                        column: 1,
                    },
                    end: {
                        offset: 29,
                        line: 5,
                        column: 1,
                    },
                },
            },
            {
                kind: "comment",
                content: " d",
                location: {
                    start: {
                        offset: 29,
                        line: 5,
                        column: 1,
                    },
                    end: {
                        offset: 33,
                        line: 6,
                        column: 1,
                    },
                },
            },
            {
                kind: "comment",
                content: " e",
                location: {
                    start: {
                        offset: 48,
                        line: 7,
                        column: 1,
                    },
                    end: {
                        offset: 52,
                        line: 8,
                        column: 1,
                    },
                },
            },
            {
                kind: "comment",
                content: " f",
                location: {
                    start: {
                        offset: 52,
                        line: 8,
                        column: 1,
                    },
                    end: {
                        offset: 56,
                        line: 9,
                        column: 1,
                    },
                },
            },
        ];
        const expected = JSON.parse(JSON.stringify(nodes));
        expected.children = [
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(0, 4),
                loc: makeLocation(1, 0, 2, 0),
                raw: "% a\n",
                value: " a",
            },
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(4, 8),
                loc: makeLocation(2, 0, 3, 0),
                raw: "% b\n",
                value: " b",
            },
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(25, 29),
                loc: makeLocation(4, 0, 5, 0),
                raw: "% c\n",
                value: " c",
            },
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(29, 33),
                loc: makeLocation(5, 0, 6, 0),
                raw: "% d\n",
                value: " d",
            },
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(48, 52),
                loc: makeLocation(7, 0, 8, 0),
                raw: "% e\n",
                value: " e",
            },
            {
                type: ast_node_types_1.ASTNodeTypes.Comment,
                range: makeRange(52, 56),
                loc: makeLocation(8, 0, 9, 0),
                raw: "% f",
                value: " f",
            },
        ];
        const actual = completeComment_1.completeComments(comments)(rawText)(nodes);
        expect(actual).toMatchObject(expected);
        ASTTester.test(actual);
    }));
    test("Not destructive while occuring an error", () => __awaiter(void 0, void 0, void 0, function* () {
        const rawText = "% a\n\\begin{document}\n% b\n\\end{document}";
        const children = [
            {
                type: ast_node_types_1.ASTNodeTypes.Str,
                range: makeRange(21, 25),
                loc: makeLocation(3, 0, 5, 0),
                raw: "% b\n",
                value: "% b",
            },
        ];
        const nodes = {
            type: ast_node_types_1.ASTNodeTypes.Document,
            range: makeRange(0, 39),
            loc: makeLocation(1, 0, 4, 14),
            raw: rawText,
            children: [
                // Dummy object. It causes error.
                {
                    type: ast_node_types_1.ASTNodeTypes.Str,
                    range: makeRange(21, 25),
                    loc: makeLocation(3, 0, 5, 0),
                    raw: "% b\n",
                    value: "% b",
                },
            ],
        };
        const comments = [
            {
                kind: "comment",
                content: " a",
                location: {
                    start: {
                        offset: 0,
                        line: 1,
                        column: 1,
                    },
                    end: {
                        offset: 4,
                        line: 2,
                        column: 1,
                    },
                },
            },
            {
                kind: "comment",
                content: " b",
                location: {
                    start: {
                        offset: 21,
                        line: 3,
                        column: 1,
                    },
                    end: {
                        offset: 25,
                        line: 4,
                        column: 1,
                    },
                },
            },
        ];
        expect(() => {
            completeComment_1.completeComments(comments)(rawText)(nodes);
        }).toThrow();
        expect(nodes.children.length).toBe(1);
        expect(nodes.children).toMatchObject(children);
    }));
});
