"use strict";
/*
 * Copyright (c) 2020 Textlint Plugin LaTex2e team. All rights reserved.
 *
 * This file is part of textlint-plugin-latex2e.
 * This software is released under the MIT License, see LICENSE.md .
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ASTTester = __importStar(require("@textlint/ast-tester"));
require("jest");
const latex_to_ast_1 = require("../src/latex-to-ast");
const kernel_1 = require("@textlint/kernel");
const ast_node_types_1 = require("@textlint/ast-node-types");
const textlint_rule_spelling_1 = __importDefault(require("textlint-rule-spelling"));
const src_1 = __importDefault(require("../src"));
const textlint_plugin_markdown_1 = __importDefault(require("@textlint/textlint-plugin-markdown"));
describe("TxtNode AST", () => {
    test("Issue 42: TypeError is occurred if `Parbreak` node appears before the first appearance of actual sentence", () => {
        const code1 = `
        \\documentclass{article}
        
        \\begin{document}
          \\begin{abstract}
            abcd
          \\end{abstract}
        \\end{document}
        `;
        const code2 = `
        \\documentclass{article}
        \\begin{document}
        
          \\begin{abstract}
            abcd
          \\end{abstract}
        \\end{document}
        `;
        ASTTester.test(latex_to_ast_1.parse(code1));
        ASTTester.test(latex_to_ast_1.parse(code2));
    });
    test("Valid ast", () => {
        const code = `
        \\documentclass{article}
        \\begin{document} Hello
        \\end{document}
        `;
        ASTTester.test(latex_to_ast_1.parse(code));
    });
    test("Parse empty comment", () => {
        const code = `\\begin{document}
        %
        hoge%
        \\end{document}
        `;
        ASTTester.test(latex_to_ast_1.parse(code));
    });
    test("Parse display math", () => {
        const code = `\\begin{equation}
          x^2 - 6x + 1 = 0
        \\end{equation}
        \\[
            x^2 -6x + 1 = 0
        \\]
        `;
        ASTTester.test(latex_to_ast_1.parse(code));
    });
    test("Parse inline math", () => {
        const code = `\\begin{document}
          $x^2 = 4$
          \\( x^2 = 4 \\)
          \\begin{math} x^2 = 4 \\end{math}
        \\end{document}
        `;
        ASTTester.test(latex_to_ast_1.parse(code));
    });
    test("Parse elements of itemize, enumerate, description as ListItem", () => {
        const code = `\\begin{document}
    \\begin{itemize}
      \\item item1
      \\item item2
      \\begin{enumerate}
        \\item item3
      \\end{enumerate}
    \\end{itemize}
    \\end{document}`;
        const parsedAst = latex_to_ast_1.parse(code);
        ASTTester.test(parsedAst);
        expect(parsedAst.children[0].children[0].type).toBe("ListItem");
        expect(parsedAst.children[0].children[2].type).toBe("ListItem");
        expect(parsedAst.children[0].children[4].type).toBe("ListItem");
        expect(parsedAst.children[0].children[4].children[0].type).toBe("ListItem");
    });
    test("Paragraph should not be nested", () => {
        const code = `\\documentclass{article}
      \\begin{document}
      \\begin{theorem}
      A Cat is cute.
      \\end{theorem}
      \\end{document}
      `;
        function isNested(isParagraph, node) {
            switch (node.type) {
                case ast_node_types_1.ASTNodeTypes.Paragraph:
                    if (isParagraph) {
                        return true;
                    }
                    else {
                        return node.children
                            .map((child) => isNested(true, child))
                            .reduce((a, b) => a || b, false);
                    }
                default:
                    if ("children" in node) {
                        return node.children
                            .map((child) => isNested(isParagraph, child))
                            .reduce((a, b) => a || b, false);
                    }
                    else {
                        return false;
                    }
            }
        }
        ASTTester.test(latex_to_ast_1.parse(code));
        expect(isNested(false, latex_to_ast_1.parse(code))).toBe(false);
    });
    test("Parse comments", () => {
        const code = `\\begin{document}
          % comment
          abcd
        \\end{document}
        `;
        const actual = latex_to_ast_1.parse(code);
        ASTTester.test(actual);
        expect(actual.children.length).toBe(3);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Comment);
        expect(actual.children[2].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
    });
    test("Parse comments (outside of document)", () => {
        const code = `
        \\documentclass{article}
        % comment
        \\begin{document}
          abcd
        \\end{document}
        % comment
        `;
        const actual = latex_to_ast_1.parse(code);
        ASTTester.test(actual);
        expect(actual.children.length).toBe(5);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Comment);
        expect(actual.children[2].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
        expect(actual.children[4].type).toBe(ast_node_types_1.ASTNodeTypes.Comment);
    });
    test("Two lines of comments between other environments", () => {
        const code = `
        \\documentclass{article}
        \\title{a}
        % first comment
        % second comment
        \\author{b}
        `;
        const actual = latex_to_ast_1.parse(code);
        ASTTester.test(actual);
        expect(actual.children.length).toBe(7);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Header);
        expect(actual.children[2].type).toBe(ast_node_types_1.ASTNodeTypes.Comment);
        expect(actual.children[4].type).toBe(ast_node_types_1.ASTNodeTypes.Comment);
    });
    test("issue52", () => {
        const code = `A%B
C`;
        const actual = latex_to_ast_1.parse(code);
        ASTTester.test(actual);
        expect(actual.children.length).toBe(1);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
        expect(actual.children[0].children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Str);
        expect(actual.children[0].children[1].type).toBe(ast_node_types_1.ASTNodeTypes.Comment);
        expect(actual.children[0].children[2].type).toBe(ast_node_types_1.ASTNodeTypes.Str);
    });
    test("comments in a equation environment are ignored", () => {
        const code = `\\begin{equation}
    % comment
    x = x^{2} % comment
\\end{equation}`;
        const actual = latex_to_ast_1.parse(code);
        ASTTester.test(actual);
        expect(actual.children.length).toBe(1);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.CodeBlock);
        expect(actual.children[0].children).toBe(undefined);
    });
    test("url command", () => {
        const code = `\\url{http://example.com/}`;
        const actual = latex_to_ast_1.parse(code);
        expect(actual.children.length).toBe(1);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
        expect(actual.children[0].children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Link);
        expect(actual.children[0].children[0].url).toBe("http://example.com/");
        expect(actual.children[0].children[0].children[0].value).toBe("http://example.com/");
    });
    test("href command", () => {
        const code = `\\href{http://example.com/}{link}`;
        const actual = latex_to_ast_1.parse(code);
        expect(actual.children.length).toBe(1);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
        expect(actual.children[0].children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Link);
        expect(actual.children[0].children[0].url).toBe("http://example.com/");
        expect(actual.children[0].children[0].children[0].value).toBe("link");
    });
    test("label command", () => {
        const code = `\\ref{label}`;
        const actual = latex_to_ast_1.parse(code);
        expect(actual.children.length).toBe(1);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Html);
        expect(actual.children[0].value).toBe("label");
        expect(actual.children[0].raw).toBe(code);
    });
    test("CodeBlock is not to be included in paragraph", () => {
        const code = `\\section{title}
hogehoge
\\begin{equation}
  x = y
\\end{equation}
fugafuga`;
        const actual = latex_to_ast_1.parse(code);
        expect(actual.children.length).toBe(7);
        expect(actual.children[0].type).toBe(ast_node_types_1.ASTNodeTypes.Header);
        expect(actual.children[2].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
        expect(actual.children[4].type).toBe(ast_node_types_1.ASTNodeTypes.CodeBlock);
        expect(actual.children[6].type).toBe(ast_node_types_1.ASTNodeTypes.Paragraph);
    });
});
describe("Fixing document", () => {
    const kernel = new kernel_1.TextlintKernel();
    const options = {
        filePath: "<test>",
        plugins: [
            { pluginId: "latex2e", plugin: src_1.default },
            {
                pluginId: "markdown",
                plugin: textlint_plugin_markdown_1.default,
            },
        ],
        rules: [
            {
                ruleId: "spelling",
                rule: textlint_rule_spelling_1.default,
                options: {
                    language: "en",
                    suggestCorrections: true,
                },
            },
        ],
    };
    test("latex code", () => __awaiter(void 0, void 0, void 0, function* () {
        const input = `
        \\documentclass{article}
        \\begin{document}
        What colour do you like?

        \\hoge
        What color do you like?
        \\end{document}
        `;
        const output = `
        \\documentclass{article}
        \\begin{document}
        What color do you like?

        \\hoge
        What color do you like?
        \\end{document}
        `;
        const result = yield kernel.fixText(input, Object.assign(Object.assign({}, options), { ext: ".tex" }));
        expect(result.output).toBe(output);
    }));
    test("latex code one line", () => __awaiter(void 0, void 0, void 0, function* () {
        const input = `
        \\documentclass{article}
        \\begin{document}What colour do you like?\\end{document}
        `;
        const output = `
        \\documentclass{article}
        \\begin{document}What color do you like?\\end{document}
        `;
        const result = yield kernel.fixText(input, Object.assign(Object.assign({}, options), { ext: ".tex" }));
        expect(result.output).toBe(output);
    }));
    test("latex code outside of document environment", () => __awaiter(void 0, void 0, void 0, function* () {
        const input = `What colour do you like?`;
        const output = `What color do you like?`;
        const result = yield kernel.fixText(input, Object.assign(Object.assign({}, options), { ext: ".tex" }));
        expect(result.output).toBe(output);
    }));
    test("latex code with comments", () => __awaiter(void 0, void 0, void 0, function* () {
        const input = `
        \\documentclass{article}
        \\begin{document}
          % comment
          What colour do you like?
        \\end{document}`;
        const output = `
        \\documentclass{article}
        \\begin{document}
          % comment
          What color do you like?
        \\end{document}`;
        const result = yield kernel.fixText(input, Object.assign(Object.assign({}, options), { ext: ".tex" }));
        expect(result.output).toBe(output);
    }));
    test("latex code with comments (outside of document)", () => __awaiter(void 0, void 0, void 0, function* () {
        // FIX: A new line before \documentclass causes an error.
        const input = `\\documentclass{article}
        % comment
        \\begin{document}
          What colour do you like?
        \\end{document}
        % comment`;
        const output = `\\documentclass{article}
        % comment
        \\begin{document}
          What color do you like?
        \\end{document}
        % comment`;
        const result = yield kernel.fixText(input, Object.assign(Object.assign({}, options), { ext: ".tex" }));
        expect(result.output).toBe(output);
    }));
    test("comments in a equation environment", () => __awaiter(void 0, void 0, void 0, function* () {
        const input = `\\begin{equation}
% Comment
x^x % comment
\\end{equation}`;
        const output = `\\begin{equation}
% Comment
x^x % comment
\\end{equation}`;
        const result = yield kernel.fixText(input, Object.assign(Object.assign({}, options), { ext: ".tex" }));
        expect(result.output).toBe(output);
    }));
});
